module Fedi.Routes.Outbox where

import Data.Aeson qualified as A
import Fedi.Helpers
import Fedi.Types
import Fedi.Types.Helpers
import Fedi.UserDetails
import Fedi.Routes.Helpers
import Web.Twain qualified as Twain
import Web.Twain.Types qualified as Twain

-- * Outbox

matchOutbox :: UserDetails -> Twain.PathPattern
matchOutbox details =
  fromString ("/" <> details.username <> "/outbox")

handleOutbox :: UserDetails -> [AnyActivity] -> Twain.ResponderM b
handleOutbox details items = do
  isPage <- Twain.queryParamMaybe "page"
  let
    outboxUrl =
      "https://"
        <> details.domain
        <> "/"
        <> details.username
        <> "/outbox"
    response =
      case isPage of
        Just True ->
          let
            empty = emptyOrderedCollectionPage outboxUrl
            content :: OutboxPage
            content =
              empty
                { id = Just $ ObjectId $ outboxUrl <> "?page=true"
                , otype =
                    empty.otype
                      { ctype =
                          empty.otype.ctype
                            { partOf = outboxUrl
                            , porderedItems = items
                            }
                      }
                }
          in
            A.encode content
        _ ->
          let
            content :: Outbox
            content =
              emptyOrderedCollection
                { id = Just $ ObjectId outboxUrl
                , summary = Just $ fromString $ details.username <> "'s notes"
                , otype =
                    emptyOrderedCollection.otype
                      { ctype =
                          emptyOrderedCollection.otype.ctype
                            { orderedItems = items
                            }
                      , first = Just $ outboxUrl <> "?page=true"
                      , last = Just $ outboxUrl <> "?page=true"
                      }
                }
          in
            A.encode content
  Twain.send $ jsonLD response
